
// $Id: AlienCommand.cpp,v 1.18 2014/02/05 08:35:47 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienCommand.h"
#include "FileInfo.h"

//___________________________________________________________________
unsigned int& AlienCommand::_counter( void )
{
    static unsigned int counter( 0 );
    return counter;
}

//___________________________________________________________________
AlienCommand::AlienCommand( const Type type ):
    Counter( "AlienCommand" ),
    id_( _counter()++ ),
    type_( type )
{}

//___________________________________________________________________
QString AlienCommand::name( const Type& type )
{
    if( _names().empty() ) _initializeNames();
    NameMap::const_iterator iter( _names().find( type ) );
    return iter != _names().end() ? iter.value():"";
}

//___________________________________________________________________
QString AlienCommand::errorString( void ) const
{

    switch( type() )
    {

        case CheckProxy:
        return QObject::tr( "No valid GSI proxy was found. "
            "All connections to the grid are likely to fail. "
            "Use <xrdgsiproxy init> to get a valid proxy." );

        case Connect:
        return QObject::tr( "Failed to connect to remote file system." );

        case HomeDirectory:
        {
            const FileInfo& path( argument(0) );
            return QString( QObject::tr( "Failed to query %1 home folder." ) ).arg( path.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) );
        }

        case ShowDirectory:
        {

            const FileInfo& path( argument(0) );
            return QString( QObject::tr( "Failed to query content of %1 folder '%2'." ) )
                .arg( path.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( path.file() );
        }

        case CreateDirectory:
        {

            const FileInfo& path( argument(0) );
            return QString( QObject::tr( "Failed to create %1 folder '%2'." ) )
                .arg( path.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( path.file()  );

        }

        case Rename:
        {

            const FileInfo& path( argument(0) );
            return QString( QObject::tr( "Failed to rename %1 %2 '%3'." ) )
                .arg( path.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( path.isFolder() ? QObject::tr( "folder" ): QObject::tr( "file" ) )
                .arg( path.file() );

        }

        case Remove:
        {

            const FileInfo& path( argument(0) );
            return QString( QObject::tr( "Failed to delete %1 %2 '%3'." ) )
                .arg( path.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( path.isFolder() ? QObject::tr( "folder" ): QObject::tr( "file" ) )
                .arg( path.file() );

        }

        case Copy:
        {
            const FileInfo& source( argument(0) );
            const FileInfo& destination( argument(1) );
            return QString( QObject::tr( "Failed to copy %1 %2 '%3' to %4 folder '%5'." ) )
                .arg( source.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( source.isFolder() ? QObject::tr( "folder" ): QObject::tr( "file" ) )
                .arg( source.file() )
                .arg( destination.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( destination.file() );

        }

        case Open:
        {

            const FileInfo& path( argument(0) );
            return QString( QObject::tr( "Failed to open %1 file '%2'." ) )
                .arg( path.isLocal() ? QObject::tr( "local" ): QObject::tr( "remote" ) )
                .arg( path.file() );

        }

        default: return QString();
    }

}

//___________________________________________________________________
void AlienCommand::_initializeNames( void )
{
    _names()[None] = QObject::tr( "none" );
    _names()[Connect] = QObject::tr( "connect" );
    _names()[CheckProxy] = QObject::tr( "check GSI proxy validity" );
    _names()[FlushErrors] = QObject::tr( "check failed commands (recursive)" );
    _names()[HomeDirectory]= QObject::tr( "query home directory" );
    _names()[ShowDirectory]= QObject::tr( "query directory content" );
    _names()[CreateDirectory]= QObject::tr( "create directory" );
    _names()[Copy]= QObject::tr( "copy selected files" );
    _names()[Locate]= QObject::tr( "locate selected files" );
    _names()[Rename]= QObject::tr( "rename selected file" );
    _names()[Remove]= QObject::tr( "delete selected files" );
    _names()[Open]= QObject::tr( "open selected files" );
    _names()[Find]= QObject::tr( "find files collection" );
}

//___________________________________________________________________
AlienCommand::NameMap& AlienCommand::_names( void )
{
    static NameMap names;
    return names;
}
