// $Id: AlienMacroModel.cpp,v 1.13 2014/11/25 14:57:22 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienMacroModel.h"
#include "AlienMacroModel.moc"

#include "CustomPixmap.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>

//_______________________________________________
const QString AlienMacroModel::columnTitles_[ AlienMacroModel::nColumns ] =
{ tr( "Name" ), tr( "File" ) };

//_______________________________________________________________
AlienMacroModel::AlienMacroModel( QObject* parent ):
    ListModel<AlienMacro>( parent ),
    Counter( "AlienMacroModel" )
{

    Debug::Throw( "AlienMacroModel::AlienMacroModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant AlienMacroModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated macro
    const AlienMacro& macro( get()[index.row()] );

    // return text associated to macro and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case Name: return macro.name();
            case Filename: return macro.argumentCount() > 0  ? macro.argument(0).file():QVariant();
            default: return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == Name )
    { return _icon(macro.type()); }

    return QVariant();

}

//__________________________________________________________________
QVariant AlienMacroModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//_________________________________________________________
void AlienMacroModel::updateConfiguration( void )
{

    Debug::Throw( "AlienMacroModel::updateConfiguration.\n" );

    beginResetModel();

    // reinitialize icons
    _icons().clear();
    _installIcons();

    endResetModel();
}

//_____________________________________________________________________
void AlienMacroModel::_installIcons( void ) const
{

    Debug::Throw( "AlienMacroList::_installIcons" );

    if( !_icons().empty() ) return;

    // type icons
    using IconNamesHash = QHash< AlienMacro::Type, QString >;
    IconNamesHash typeNames;
    typeNames[AlienMacro::Add] = IconNames::Add;
    typeNames[AlienMacro::Clear] = IconNames::Delete;
    typeNames[AlienMacro::Connect] = IconNames::Connect;
    typeNames[AlienMacro::CheckProxy] = IconNames::Connect;
    typeNames[AlienMacro::HomeDirectory] = IconNames::Home;
    typeNames[AlienMacro::HomeDirectory] = IconNames::Home;
    typeNames[AlienMacro::CreateDirectory] = IconNames::CreateDirectory;
    typeNames[AlienMacro::ShowDirectory] = IconNames::Find;
    typeNames[AlienMacro::Copy] = IconNames::Paste;
    typeNames[AlienMacro::Rename] = IconNames::Rename;
    typeNames[AlienMacro::Remove] = IconNames::Delete;
    typeNames[AlienMacro::Open] = IconNames::Edit;
    typeNames[AlienMacro::Find] = IconNames::Find;
    typeNames[AlienMacro::ListRecursive] = IconNames::Find;
    typeNames[AlienMacro::SizeRecursive] = IconNames::Find;
    typeNames[AlienMacro::CopyRecursive] = IconNames::Paste;
    typeNames[AlienMacro::RemoveRecursive] = IconNames::Paste;

    // load link pixmap
    for( IconNamesHash::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}

//_______________________________________________
AlienMacroModel::IconCache& AlienMacroModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
const QIcon& AlienMacroModel::_icon( AlienMacro::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter != _icons().end() ) return iter.value();
    else return _icons().insert( type, CustomPixmap( IconSize( IconSize::Medium ), CustomPixmap::Transparent ) ).value();

}

//________________________________________________________
bool AlienMacroModel::SortFTor::operator () ( AlienMacro first, AlienMacro second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {

        case AlienMacroModel::Name:
        if( first.type() != second.type() ) return first.type() < second.type();
        else break;

        case AlienMacroModel::Filename:
        {
            const FileInfo source( first.argumentCount() > 0 ? first.argument(0).file():FileInfo() );
            const FileInfo destination( second.argumentCount() > 0 ? second.argument(0).file():FileInfo() );
            if( source.file() != destination.file() ) return source.file() < destination.file();
            else break;
        }

        default: break;
    }

    return first.id() < second.id();

}
