// $Id: Application.cpp,v 1.55 2014/03/11 22:10:30 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "Application.moc"

#include "AlienDefines.h"
#include "Config.h"
#include "ConfigurationDialog.h"
#include "FileSelectionFrame.h"
#include "MainWindow.h"
#include "ProgressStatusBar.h"
#include "QuestionDialog.h"
#include "ScratchFileMonitor.h"
#include "TreeView.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>

//____________________________________________
Application::Application( CommandLineArguments arguments ):
    BaseApplication( 0, arguments ),
    Counter( "Application" ),
    mainWindow_( 0 )
{ Debug::Throw( "Application::Application.\n" ); }

//____________________________________________
Application::~Application( void )
{}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // top widget
    mainWindow_ = new MainWindow();
    mainWindow_->setWindowTitle( "Gridftp" );

    // scratch files
    scratchFileMonitor_ = new ScratchFileMonitor( this );
    connect( qApp, SIGNAL(aboutToQuit()), scratchFileMonitor_, SLOT(deleteScratchFiles()) );
    connect( &mainWindow(), SIGNAL(scratchFileCreated(File)), scratchFileMonitor_, SLOT(add(File)) );

    // show main window
    mainWindow_->centerOnDesktop();
    mainWindow_->show();

    // parse arguments
    QStringList filenames( commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) mainWindow_->initialize( filenames.front() );
    else mainWindow_->initialize();

    // configuration
    emit configurationChanged();

    return true;

}

//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );
    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( CommandLineParser::Tag( "--force", "-f" ), tr( "unconditionally select command-line path in existing window") );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void Application::usage( void ) const
{
    _usage( "gridftp", tr( "[options] [path]" ) );
    commandLineParser().usage();
    return;
}

//___________________________________________________________
void Application::_configuration( void )
{
    Debug::Throw( "Application::_configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();
}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow_->uniconify();

        const CommandLineParser parser( commandLineParser( command.arguments() ) );
        const QStringList filenames( parser.orphans() );
        if( filenames.isEmpty() ) return true;
        else if( parser.hasFlag( "--force" ) ) mainWindow_->updatePath( filenames.front() );
        else {
            const QString buffer = QString( tr( "Change %1 directory to path '%2' ?" ) )
                .arg( Alien::isRemote( filenames.front() ) ? tr( "remote" ):tr( "local" ) )
                .arg( filenames.front() );
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            { mainWindow_->updatePath( filenames.front() ); }
        }

        return true;

    } else return false;

}
