// $Id: ConfigurationDialog.cpp,v 1.29 2014/09/26 08:06:18 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConfigurationDialog.h"
#include "ConfigurationDialog.moc"

#include "BaseFileInfoConfigurationWidget.h"
#include "GridLayout.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "OptionBrowsedLineEditor.h"
#include "OptionCheckBox.h"
#include "OptionListBox.h"
#include "OptionLineEditor.h"
#include "OptionSpinBox.h"
#include "PathHistoryConfiguration.h"

#include <QGroupBox>
#include <QLabel>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
    BaseConfigurationDialog( parent )
{

    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( "Configuration - Gridwatch" );

    // base configuration
    baseConfiguration( 0 );

    QWidget* page;
    QWidget *box;
    GridLayout* gridLayout;

    // appearance
    page = &addPage( IconEngine::get( IconNames::PreferencesAppearance), tr( "Appearance" ), tr( "Window appearance and colors" ) );
    box = new QWidget( page );
    page->layout()->addWidget( box );

    QVBoxLayout* layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    box->setLayout( layout );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    layout->addLayout( gridLayout );

    QLabel* label;
    gridLayout->addWidget( label = new QLabel( tr( "Icon size in icon view:" ), box ) );
    OptionSpinBox *spinbox = new OptionSpinBox( box, "ICON_VIEW_PIXMAP_SIZE" );
    spinbox->setMinimum( 8 );
    spinbox->setMaximum( 128 );
    spinbox->setSuffix( tr( "px" ) );
    gridLayout->addWidget( spinbox );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // prefix
    OptionCheckBox* checkbox;
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Show file system prefix inside location bar" ), box, "USE_PREFIX" ) );
    addOptionWidget( checkbox );

    // full path
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Show full path inside location bar" ), box, "USE_TRUNCATION" ) );
    checkbox->setNegative( true );
    addOptionWidget( checkbox );

    // navigation
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Show parent directory icon in file list" ), box, "SHOW_NAVIGATOR" ) );
    addOptionWidget( checkbox );

    // hidden files
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Show hidden files" ), box, "SHOW_HIDDEN_FILES" ) );
    addOptionWidget( checkbox );

    // tooltips
    page = &addPage( IconEngine::get( IconNames::PreferencesAppearance), tr( "Tooltips" ), tr( "Tooltips appearance" ) );
    page->layout()->addWidget( checkbox = new OptionCheckBox( tr( "Show tooltips" ), page, "SHOW_TOOLTIPS" ) );
    addOptionWidget( checkbox );

    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(5);
    hLayout->setSpacing(0);
    page->layout()->addItem( hLayout );

    BaseFileInfoConfigurationWidget* configurationWidget;
    hLayout->addSpacing( _checkBoxSpacing() );
    hLayout->addWidget( configurationWidget = new BaseFileInfoConfigurationWidget( page ) );
    configurationWidget->setEnabled( false );
    connect( checkbox, SIGNAL(toggled(bool)), configurationWidget, SLOT(setEnabled(bool)) );
    addOptionWidget( configurationWidget );

    // commands
    page = &addPage( IconEngine::get( IconNames::PreferencesFileAssociations ), tr( "Applications" ), tr( "Third-party applications" ) );
    box = new QGroupBox( page );
    page->layout()->addWidget( box );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    box->setLayout( gridLayout );

    OptionBrowsedLineEditor* editor;
    gridLayout->addWidget( label = new QLabel( tr( "Proxy configuration:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "GSI_PROXY" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    gridLayout->addWidget( label = new QLabel( tr( "Alien copy:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "ALIEN_CP" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    gridLayout->addWidget( label = new QLabel( tr( "Busy box:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "GBBOX" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    box = new QGroupBox( tr( "Environment" ), page );
    page->layout()->addWidget( box );
    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    box->setLayout( gridLayout );

    gridLayout->addWidget( label = new QLabel( tr( "Certificates:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "CERTIFICATES_PATH" ) );
    editor->setFileMode( QFileDialog::Directory );
    addOptionWidget( editor );
    label->setBuddy( editor );

    OptionLineEditor* hostEditor;
    gridLayout->addWidget( label = new QLabel( tr( "Connection host:" ), box ) );
    gridLayout->addWidget( hostEditor = new OptionLineEditor( box, "ALIEN_HOST" ) );
    addOptionWidget( hostEditor );
    label->setBuddy( hostEditor );

    gridLayout->addWidget( label = new QLabel( tr( "Connection port:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "ALIEN_PORT" ) );
    spinbox->setMinimum(0);
    spinbox->setMaximum( INT_MAX );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // external applications
    box = new QGroupBox( tr( "Applications Used to Open Files" ), page );
    page->layout()->addWidget( box );

    QVBoxLayout* vLayout = new QVBoxLayout();
    vLayout->setSpacing(5);
    vLayout->setMargin(5);
    box->setLayout( vLayout );

    OptionListBox *listbox = new OptionListBox( box, "APPLICATIONS" );
    listbox->setBrowsable( true );
    listbox->setFileMode( QFileDialog::ExistingFiles );
    listbox->setToolTip( tr( "List of applications used to open files" ) );
    vLayout->addWidget( listbox );
    addOptionWidget( listbox );

    // unsorted
    page = &addPage( IconEngine::get( IconNames::PreferencesUnsorted ), tr( "Unsorted" ), tr( "Additional unsorted settings" ) );
    box = new QWidget( page );
    page->layout()->addWidget( box );

    layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    box->setLayout( layout );

    // restore session
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Restore previous session at startup" ), box, "RESTORE_SESSION" ) );
    addOptionWidget( checkbox );

    // log
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Enable command log" ), box, "LOG_ENABLED" ) );
    addOptionWidget( checkbox );

    // history
    PathHistoryConfiguration* pathHistoryConfiguration;
    layout->addWidget( pathHistoryConfiguration = new PathHistoryConfiguration( box ) );
    addOptionWidget( pathHistoryConfiguration );

    // load initial configuration
    read();

}
