// $Id: FileInfoModel.cpp,v 1.49 2014/04/02 19:19:32 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "FileInfoModel.h"
#include "FileInfoModel.moc"

#include "AlienDefines.h"
#include "FileIconProvider.h"
#include "TimeStamp.h"
#include "XmlOptions.h"
#include "QOrderedSet.h"

#include <QUrl>
#include <QIcon>

//_______________________________________________________________
FileInfoModel::FileInfoModel( QObject* parent, BaseFileInfo::TypeFlags type ):
    BaseFileInfoModel<FileInfo>( parent ),
    Counter( "FileInfoModel" ),
    type_( type ),
    iconProvider_( 0x0 )
{ Debug::Throw( "FileInfoModel::FileInfoModel.\n" ); }

//__________________________________________________________________
Qt::ItemFlags FileInfoModel::flags(const QModelIndex &index) const
{

    Qt::ItemFlags flags( BaseFileInfoModel<FileInfo>::flags( index ) );
    if( !index.isValid() )
    {

        flags |= Qt::ItemIsDropEnabled;

    } else if( contains( index ) ) {

        // retrieve associated file info
        const FileInfo& fileInfo( get()[index.row()] );
        if( fileInfo.isNavigator() ) flags &= ~Qt::ItemIsSelectable;
        else {

            flags |= Qt::ItemIsDragEnabled;
            if( fileInfo.isFolder() ) flags |= Qt::ItemIsDropEnabled;

        }

    }

    return flags;

}

//__________________________________________________________________
void FileInfoModel::setClipboard( FileSelectionClipboard clipboard )
{

    // remove files that are not of same local type
    if( clipboard.mode() == FileSelectionClipboard::Cut )
    {

        FileSelectionClipboard::iterator iter( clipboard.begin() );
        while( iter != clipboard.end() )
        {
            if( iter->isLocal() != _isLocal() ) iter = clipboard.erase( iter );
            else ++iter;
        }

    } else clipboard.clear();

    // do nothing if no actual change
    if( clipboard_ == clipboard ) return;

    // if current is empty and new is either invalid or empty do nothing
    if( clipboard_.isEmpty() && clipboard.isEmpty() ) return;

    // update
    emit layoutAboutToBeChanged();
    clipboard_ = clipboard;
    emit layoutChanged();

}

//__________________________________________________________________
QVariant FileInfoModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated file info
    const FileInfo& fileInfo( get()[index.row()] );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case Location:
            return QString( fileInfo.isLocal() ? tr( "Local" ): tr( "Remote" ) );

            case Status:
            return QString( fileInfo.isSuccess() ? tr( "Success" ): tr( "Failed" ) );

            case Action:
            return QString( fileInfo.isOverwrite() ? tr( "Overwrite" ): tr( "Skip" ) );

            default:
            return BaseFileInfoModel<FileInfo>::data( index, role );
        }
    }

    return BaseFileInfoModel<FileInfo>::data( index, role );

}

//______________________________________________________________________
QStringList FileInfoModel::mimeTypes( void ) const
{
    QStringList types;
    if( _isRemote() ) types << BaseFileInfo::MimeType << "text/uri-list";
    if( _isLocal() ) types << BaseFileInfo::MimeType;
    return types;
}

//______________________________________________________________________
QMimeData* FileInfoModel::mimeData(const QModelIndexList &indexes) const
{

    QOrderedSet<QString> filenames;
    QOrderedSet<FileInfo> fileInfoList;

    foreach( const QModelIndex& index, indexes )
    {

        if( index.isValid() && (flags( index )&Qt::ItemIsDragEnabled) )
        {
            FileInfo info( get(index) );

            fileInfoList.insert( info );
            filenames.insert( info.file() );
        }

    }

    if( filenames.empty() ) return 0;
    else {

        QMimeData *mimeData = new QMimeData();

        // fill text and url data
        QList<QUrl> urlList;
        QStringList fileList;
        foreach( const QString& filename, filenames )
        {
            const QString url( ( _isLocal() ? Alien::localPrefix : Alien::remotePrefix ) + "//" + filename );
            urlList << url;
            fileList << url;
        }

        mimeData->setText( fileList.join( "\n" ) );
        if( _isLocal() ) mimeData->setUrls( urlList );

        // fill Drag data. Use XML
        QDomDocument document;
        QDomElement top = document.appendChild( document.createElement( Xml::FileInfoList ) ).toElement();
        foreach( const FileInfo& fileInfo, fileInfoList )
        {

            if( fileInfo.file().isEmpty() ) continue;
            top.appendChild( fileInfo.domElement( document ) );

        }

        mimeData->setData( BaseFileInfo::MimeType, document.toByteArray() );
        return mimeData;

    }

}

//__________________________________________________________________
bool FileInfoModel::dropMimeData(const QMimeData* data , Qt::DropAction action, int, int, const QModelIndex& parent)
{

    Debug::Throw( "FileInfoModel::dropMimeData.\n" );

    // check action
    if( action == Qt::IgnoreAction) return true;

    // info list
    FileInfoModel::List fileInfoList;

    // check format
    if( data->hasFormat( BaseFileInfo::MimeType ) )
    {

        // get dropped file info (use XML)
        // dom document
        QDomDocument document;
        if( !document.setContent( data->data( BaseFileInfo::MimeType ), false ) ) return false;

        QDomElement docElement = document.documentElement();
        QDomNode node = docElement.firstChild();
        for(QDomNode node = docElement.firstChild(); !node.isNull(); node = node.nextSibling() )
        {
            QDomElement element = node.toElement();
            if( element.isNull() ) continue;

            // special options
            if( element.tagName() == Xml::FileInfo )
            {

                const FileInfo fileInfo( element );
                if( fileInfo.file().isEmpty() || fileInfo.isNavigator() ) continue;
                fileInfoList << fileInfo;

            }
        }

    } else if( _isRemote() && data->hasUrls() ) {

        const QList<QUrl> urls( data->urls() );
        foreach( const QUrl& url, urls )
        {

            #if QT_VERSION >= 0x040800
            // check that local file
            if( !url.isLocalFile() ) continue;
            #endif

            // get file and check existence
            const File file( url.path() );
            if( !file.exists() ) continue;

            FileInfo fileInfo( file );
            fileInfo.update();

            fileInfo.setLocal();
            if( file.isDirectory() ) fileInfo.setIsFolder();
            else fileInfo.setIsDocument();
            if( file.isLink() ) fileInfo.setIsLink();
            if( file.isBrokenLink() ) fileInfo.setIsBrokenLink();
            if( file.isHidden() ) fileInfo.setIsHidden();

            fileInfoList << fileInfo;

        }

    }

    if( fileInfoList.isEmpty() ) return false;

    // check destination
    FileInfo destination;
    if( parent.isValid() && get( parent ).isFolder() ) destination = get( parent );
    else destination.setType( type_ );

    // delayed transfer request emission
    // this fixes issues with nested event loops
    transferData_.set( destination, fileInfoList );
    QMetaObject::invokeMethod( this, "_requestTransfer", Qt::QueuedConnection );

    return true;

}

//__________________________________________________________________
QVariant FileInfoModel::headerData(int column, Qt::Orientation orientation, int role) const
{

    if( orientation == Qt::Horizontal && role == Qt::DisplayRole )
    {
        switch( column )
        {
            case Location: return tr( "Location" );
            case Status: return tr( "Status" );
            case Action: return tr( "Action" );
            default: break;
        }
    }

    // base class implementation
    return BaseFileInfoModel<FileInfo>::headerData( column, orientation, role );

}

//____________________________________________________________
void FileInfoModel::_sort( int column, Qt::SortOrder order )
{ std::sort( _get().begin(), _get().end(), SortFTor( (ColumnType) column, order ) ); }

//____________________________________________________________
QIcon FileInfoModel::_icon( const QModelIndex& index ) const
{

    // check provider
    if( !iconProvider_ ) return QIcon();

    // get file info and check if in clipboard
    FileInfo fileInfo( get( index ) );
    int type( fileInfo.type() );
    if( clipboard_.mode() == FileSelectionClipboard::Cut && clipboard_.contains( fileInfo ) )
    { type |= FileInfo::Clipped; }

    // return icon from provider
    return iconProvider_->icon( fileInfo, type );

}

//________________________________________________________
bool FileInfoModel::SortFTor::operator () ( const FileInfo& constFirst, const FileInfo& constSecond ) const
{

    // Navigator come first
    // Folders come next
    if( constFirst.isNavigator() ) return true;
    else if( constSecond.isNavigator() ) return false;
    else if(
        (constFirst.type()&(BaseFileInfo::Folder|BaseFileInfo::Document)) !=
        (constSecond.type()&(BaseFileInfo::Folder|BaseFileInfo::Document)) )
    { return constFirst.isFolder(); }

    FileInfo first( constFirst );
    FileInfo second( constSecond );
    if( order_ == Qt::DescendingOrder ) std::swap( first, second );

    switch( type_ )
    {

        case Location:
        {
            if( first.isLocal() && second.isRemote() ) return true;
            else if( second.isLocal() && first.isRemote() ) return false;
            else return first.file() < second.file();
        }

        case Status:
        return first.isSuccess() && !second.isSuccess();

        case Action:
        return first.isOverwrite() && !second.isOverwrite();

        default:
        return BaseFileInfoModel<FileInfo>::SortFTor::operator() ( constFirst, constSecond );

    }

}
