// $Id: FindCollectionDialog.cpp,v 1.25 2014/09/26 08:06:18 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "FindCollectionDialog.h"
#include "FindCollectionDialog.moc"

#include "AnimatedTextEditor.h"
#include "CustomComboBox.h"
#include "ElidedLabel.h"
#include "FileInfo.h"
#include "FileDialog.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "InformationDialog.h"
#include "XmlOptions.h"

#include <QToolButton>

//_________________________________________________
FindCollectionDialog::FindCollectionDialog( QWidget* parent ):
    CustomDialog( parent, CloseButton )
{

    Debug::Throw( "FindCollectionDialog::FindCollectionDialog.\n" );
    setOptionName( "FIND_COLLECTION_DIALOG" );

    // grid layout
    GridLayout* gridLayout( new GridLayout() );
    gridLayout->setSpacing(5);
    gridLayout->setMargin(0);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment(0, Qt::AlignVCenter|Qt::AlignRight );
    mainLayout().addLayout( gridLayout );

    QLabel* label;

    // path
    gridLayout->addWidget( label = new QLabel( tr( "Path:" ), this ), 0, 0, 1, 1 );
    gridLayout->addWidget( pathLabel_ = new ElidedLabel( this ), 0, 1, 1, 2 );

    {
        const Qt::TextInteractionFlags defaultFlags( QLabel().textInteractionFlags() );
        pathLabel_->setTextInteractionFlags( Qt::TextSelectableByMouse | defaultFlags );
    }

    // pattern
    gridLayout->addWidget( label = new QLabel( tr( "File pattern:" ), this ), 1, 0, 1, 1 );
    gridLayout->addWidget( patternComboBox_ = new CustomComboBox( this ), 1, 1, 1, 1 );
    label->setBuddy( patternComboBox_ );
    patternComboBox_->setEditable( true );
    patternComboBox_->setAutoCompletion( true, Qt::CaseSensitive );

    // clear history
    QToolButton* button = new QToolButton( this );
    button->setAutoRaise( true );
    button->setIcon( IconEngine::get( IconNames::ClearHistory ) );
    button->setToolTip( tr( "Clear files history" ) );
    gridLayout->addWidget( button, 1, 2, 1, 1 );

    connect( button, SIGNAL(clicked()), SLOT(_clearHistory()) );

    // retrieve applications from options
    foreach( const Option& application, XmlOptions::get().specialOptions( "COLLECTION_SEARCH_STRING" ) )
    { patternComboBox_->addItem( application.raw() ); }

    // type
    gridLayout->addWidget( label = new QLabel( tr( "Output format:" ), this ), 2, 0, 1, 1 );
    gridLayout->addWidget( comboBox_ = new QComboBox( this ), 2, 1, 1, 1 );
    label->setBuddy( comboBox_ );
    comboBox_->setEditable( false );
    comboBox_->addItems( QStringList() << tr( "Flat" ) << tr( "Xml" ) );

    if( XmlOptions::get().contains( "DEFAULT_FIND_MODE" ) )
    { comboBox_->setCurrentIndex( XmlOptions::get().get<int>( "DEFAULT_FIND_MODE" ) ); }

    gridLayout->setColumnStretch( 1, 1 );

    // editor
    mainLayout().addWidget( editor_ = new AnimatedTextEditor( this ) );
    editor_->setReadOnly( true );

    // save button
    saveButton_ = new QPushButton( IconEngine::get( IconNames::SaveAs ), tr( "Save As..." ), this );
    connect( saveButton_, SIGNAL(clicked()), SLOT(_save()) );
    buttonLayout().insertWidget(1, saveButton_ );

    // find button
    findButton_ = new QPushButton( IconEngine::get( IconNames::Find ), tr( "Find" ), this );
    connect( findButton_, SIGNAL(clicked()), SLOT(_requestUpdate()) );
    buttonLayout().insertWidget(1, findButton_ );
    findButton_->setDefault( true );
    findButton_->setAutoDefault( true );

    // status label
    buttonLayout().insertWidget( 0, statusLabel_ = new QLabel( this ) );

    // connections
    connect( patternComboBox_, SIGNAL(editTextChanged(QString)), SLOT(_updateButtons()) );
    connect( comboBox_, SIGNAL(currentIndexChanged(QString)), SLOT(_updateButtons()) );
    connect( editor_, SIGNAL(textChanged()), SLOT(_updateButtons()) );

    patternComboBox_->setFocus();

}

//_________________________________________________
void FindCollectionDialog::setPattern( const QString& value )
{ patternComboBox_->setEditText( value ); }

//_________________________________________________
void FindCollectionDialog::processCollection( const QString& constValue )
{

    Debug::Throw( "FindCollectionDialog::processCollection.\n" );
    QString value( constValue );

    // remove useless leading line (alien bug)
    QRegExp regExp( "AND THE OPTIONS ARE\\s*" );
    int position( value.indexOf( regExp ) );
    if( position >= 0 ) value = value.mid( position+regExp.matchedLength() );

    // remove trailing spaces at end of string (alien bug)
    if( comboBox_->currentIndex() == 1 ) value.replace( QRegExp( "\\s+$" ), "" );

    // assign text to editor
    editor_->setText( value );

    // count number of entries to assign to label
    int fileCount( 0 );
    if( comboBox_->currentIndex() == 0 )
    {

        QRegExp regExp( "(\\d+) files found" );
        if( value.indexOf( regExp ) >= 0 )
        { fileCount = regExp.cap(1).toInt(); }

    } else {

        // count number of occurences of <event name="XXX"> in string
        fileCount = value.count( QRegExp( "<event name=\"\\d+\">" ) );

    }

    if( fileCount == 0 ) statusLabel_->setText( tr( "No match found" ) );
    else if( fileCount == 1 ) statusLabel_->setText( tr( "Found one matching file" ) );
    else statusLabel_->setText( QString( tr( "Found %1 matching files" ) ).arg( fileCount ) );

}

//_________________________________________________
void FindCollectionDialog::_clearHistory( void )
{
    Debug::Throw( "FindCollectionDialog::_clearHistory.\n" );

    // clear combobox
    while( patternComboBox_->QComboBox::count() > 0 )
    { patternComboBox_->removeItem(0); }

    // clear option
    XmlOptions::get().clearSpecialOptions( "COLLECTION_SEARCH_STRING" );

}

//_________________________________________________
void FindCollectionDialog::_requestUpdate( void )
{

    Debug::Throw( "FindCollectionDialog::_requestUpdate.\n" );
    const QString pattern( patternComboBox_->currentText() );
    if( path_.isEmpty() || pattern.isEmpty() ) return;
    emit macroRequested( AlienMacro( AlienMacro::Find )
        << FileInfo( path_, FileInfo::Remote|FileInfo::Folder )
        << FileInfo( pattern )
        << FileInfo( QString::number( comboBox_->currentIndex() ) ) );

    // save options
    XmlOptions::get().set<int>( "DEFAULT_FIND_MODE", comboBox_->currentIndex() );
    XmlOptions::get().add( "COLLECTION_SEARCH_STRING", Option( pattern, Option::Recordable|Option::Current ) );

    // disable button
    findButton_->setEnabled( false );

}

//_________________________________________________
void FindCollectionDialog::_save( void )
{
    Debug::Throw( "FindCollectionDialog::_save.\n" );
    const QString text( editor_->toPlainText() );
    if( text.isEmpty() ) return;

    // get file
    FileDialog dialog( this );
    dialog.setFileMode( QFileDialog::AnyFile );
    dialog.setAcceptMode( QFileDialog::AcceptSave );
    if( !filename_.isEmpty() ) dialog.selectFile( filename_ );

    File file( dialog.getFile() );
    if( file.isNull() ) return;
    else file = file.expand();

    // check if file is directory
    if( file.isDirectory() )
    {
        const QString buffer = QString( tr( "File '%1' is a directory. <Save> canceled." ) ).arg( file );
        InformationDialog( this, buffer ).exec();
        return;
    }

    // open output file
    QFile out( file );
    if( !out.open( QIODevice::WriteOnly ) )
    {
        const QString buffer = QString( tr( "Cannot write to file '%1'. <Save> canceled." ) ).arg( file );
        InformationDialog( this, buffer ).exec();
        return;
    }

    out.write( text.toLatin1() );
    if( !text.isEmpty() && text[text.size()-1] != '\n' ) out.write( "\n" );

    // close
    out.close();

    // reassign filename
    filename_ = file;

}


//_________________________________________________
void FindCollectionDialog::_updateButtons( void )
{
    saveButton_->setEnabled( !editor_->toPlainText().isEmpty() );
    findButton_->setEnabled( !(path_.isEmpty() || patternComboBox_->currentText().isEmpty() ) );
    statusLabel_->clear();
}
