// $Id: MainWindow.cpp,v 1.110 2014/09/26 08:06:18 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"
#include "MainWindow.moc"

#include "AlienCommandDialog.h"
#include "AlienDefines.h"
#include "AlienInterface.h"
#include "AnimatedTreeView.h"
#include "Application.h"
#include "BusyWidget.h"
#include "CommandsFailedDialog.h"
#include "CopyFilesDialog.h"
#include "CustomToolBar.h"
#include "DockWidget.h"
#include "FileIconProvider.h"
#include "FileInformationDialog.h"
#include "FileSelectionFrame.h"
#include "FindCollectionDialog.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "LogDialog.h"
#include "LogWidget.h"
#include "Menu.h"
#include "OpenFilesDialog.h"
#include "PathEditor.h"
#include "PlacesWidget.h"
#include "ProgressStatusBar.h"
#include "QuestionDialog.h"
#include "Singleton.h"
#include "Util.h"
#include "ViewModeMenu.h"
#include "WarningDialog.h"
#include "XmlOptions.h"

#include <QDir>

#include <QApplication>
#include <QHeaderView>
#include <QLayout>
#include <QSplitter>

//____________________________________________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    Counter( "MainWindow" ),
    showNavigator_( true ),
    wasConnected_( false )
{

    Debug::Throw( "MainWindow::MainWindow.\n" );
    setOptionName( "MAIN_WINDOW" );

    // interface
    interface_ = new AlienInterface( this );
    interface_->findCertificates();

    // dialogs
    commandDialog_ = new AlienCommandDialog();
    logDialog_ = new LogDialog();

    connect( commandDialog_, SIGNAL(dialogHidden()), SLOT(_commandDialogHidden()) );
    connect( logDialog_, SIGNAL(dialogHidden()), SLOT(_logDialogHidden()) );

    // actions
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    QVBoxLayout* mainLayout( new QVBoxLayout() );
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    mainLayout->setSpacing(2);
    setCentralWidget( main );

    QHBoxLayout* hLayout( new QHBoxLayout() );
    hLayout->setMargin(0);
    hLayout->setSpacing(2);
    mainLayout->addLayout( hLayout );

    // splitter
    QSplitter *splitter;
    hLayout->addWidget( splitter = new QSplitter( main ) );

    // create frames
    splitter->addWidget( localSelectionFrame_ = new FileSelectionFrame( main, FileInfo::None ) );
    splitter->addWidget( remoteSelectionFrame_ = new FileSelectionFrame( main, FileInfo::Remote ) );

    // create icon provider and assign
    iconProvider_ = new FileIconProvider( this );
    foreach( FileSelectionFrame* frame, selectionFrames() )
    { frame->model().setIconProvider( iconProvider_ ); }

    // status bar
    setStatusBar( statusbar_ = new ProgressStatusBar( this ) );
    statusbar_->addClock();

    // toolbars
    CustomToolBar* toolbar =  new CustomToolBar( tr( "Main Toolbar" ), this, "TOOLBAR" );
    toolbar->addAction( homeDirectoryAction_ );
    toolbar->addWidget( previousDirectoryButton_ );
    toolbar->addWidget( nextDirectoryButton_ );
    toolbar->addAction( parentDirectoryAction_ );
    toolbar->addSeparator();
    toolbar->addAction( reloadAction_ );
    toolbar->addAction( abortAction_ );

    // dock widgets
    placesPanel_ = new DockWidget( tr( "Places" ), this, "PLACES" );
    placesPanel_->setAllowedAreas( Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea );
    placesPanel_->setUseScrollArea( true );
    placesPanel_->visibilityAction().setText( tr( "Show Favorite Places" ) );
    addDockWidget( Qt::LeftDockWidgetArea, placesPanel_ );

    placesWidget_ = new PlacesWidget();
    placesWidget_->setIconProvider( iconProvider_ );
    placesPanel_->setMainWidget( placesWidget_ );

    if( !placesPanel_->visibilityAction().isChecked() )
    { placesPanel_->hide(); }

    // menu bar
    Menu* menu( new Menu( this ) );
    setMenuBar( menu );
    connect( &menu->viewModeMenu(), SIGNAL(viewModeSelected(FileSelectionFrame::ViewMode)), SLOT(_updateViewMode(FileSelectionFrame::ViewMode)) );

    // connections
    foreach( FileSelectionFrame* frame, selectionFrames() )
    {
        connect( frame, SIGNAL(fileSelected(FileInfo)), SLOT(_fileSelected(FileInfo)) );
        connect( frame, SIGNAL(actionsChanged()), SLOT(_updateNavigationActions()) );
        connect( frame, SIGNAL(clipboardChanged(FileSelectionClipboard)), SLOT(_updateClipboard(FileSelectionClipboard)) );
        connect( frame, SIGNAL(pasteClipboardRequested(FileInfo)), SLOT(_pasteClipboard(FileInfo)) );
        connect( frame, SIGNAL(openFilesRequested(const FileInfo::List&)), SLOT(_openFiles(const FileInfo::List&)) );

        connect( &frame->model(), SIGNAL(transferRequested(FileInfo,const FileInfo::List&)), SLOT(_pasteSelection(FileInfo,const FileInfo::List&)) );
        connect( &frame->visibilityAction(), SIGNAL(toggled(bool)), SLOT(_updateVisibilityActions()) );

        // interface connections
        connect( frame, SIGNAL(macroRequested(AlienMacro)), interface_, SLOT(addMacro(AlienMacro)) );
        connect( frame, SIGNAL(macrosRequested(const AlienMacro::List&)), interface_, SLOT(addMacros(const AlienMacro::List&)) );
        connect( frame, SIGNAL(requestFileInformation(FileInfo,QIcon)), SLOT(_fileInformation(FileInfo,QIcon)) );
    }

    connect( remoteSelectionFrame_, SIGNAL(collectionRequested(FileInfo)), SLOT(_findCollection(FileInfo)) );

    // places
    connect( placesWidget_, SIGNAL(itemSelected(BaseFileInfo)), SLOT(_placeSelected(BaseFileInfo)) );
    connect( placesWidget_, SIGNAL(remoteItemAdded(BaseFileInfo)), SLOT(_requestPlacesUpdate(BaseFileInfo)) );

    // interface connections
    connect( this, SIGNAL(macroRequested(AlienMacro)), interface_, SLOT(addMacro(AlienMacro)) );
    connect( this, SIGNAL(macrosRequested(const AlienMacro::List&)), interface_, SLOT(addMacros(const AlienMacro::List&)) );

    connect( interface_, SIGNAL(homeAvailable(FileInfo,bool)), this, SLOT(_updateHome(FileInfo,bool)) );
    connect( interface_, SIGNAL(contentAvailable(FileInfo,const FileInfo::List&)), this, SLOT(_updateContent(FileInfo,const FileInfo::List&)) );
    connect( interface_, SIGNAL(fileInfoAdded(FileInfo)), this, SLOT(_addFileInfo(FileInfo)) );
    connect( interface_, SIGNAL(fileInfoRemoved(FileInfo)), this, SLOT(_removeFileInfo(FileInfo)) );
    connect( interface_, SIGNAL(commandsFailed(const AlienCommand::List&)), this, SLOT(_commandsFailed(const AlienCommand::List&)) );

    connect( interface_, SIGNAL(messageAvailable(QString)), &statusbar_->label(), SLOT(setTextAndUpdate(QString)) );
    connect( interface_, SIGNAL(ready()), &statusbar_->label(), SLOT(clear()) );

    connect( interface_, SIGNAL(macroStarted(AlienMacro)), this, SLOT(_macroStarted(AlienMacro)) );
    connect( interface_, SIGNAL(macroCompleted(AlienMacro)), this, SLOT(_macroCompleted(AlienMacro)) );
    connect( interface_, SIGNAL(maxProgressAvailable(int)), &statusbar_->progressBar(), SLOT(setMaximum(int)) );
    connect( interface_, SIGNAL(progressAvailable(int)), &statusbar_->progressBar(), SLOT(setValue(int)) );

    // connection state
    connect( interface_, SIGNAL(connectionStateChanged(bool)), this, SLOT(_connectionStateChanged(bool)) );
    connect( interface_, SIGNAL(connectionLost(QString)), this, SLOT(_lostConnection(QString)) );

    // busy state
    connect( interface_, SIGNAL(busyStateChanged(bool)), abortAction_, SLOT(setEnabled(bool)) );
    connect( interface_, SIGNAL(busy()), this, SLOT(_busy()) );
    connect( interface_, SIGNAL(ready()), this, SLOT(_idle()) );

    // scratch files
    connect( interface_, SIGNAL(scratchFileCreated(File)), this, SIGNAL(scratchFileCreated(File)) );

    // macro model connections
    connect( interface_, SIGNAL(macroAdded(AlienMacro)), &commandDialog_->macroModel(), SLOT(add(AlienMacro)) );
    connect( interface_, SIGNAL(macrosAdded(const AlienMacro::List&)), &commandDialog_->macroModel(), SLOT(add(const AlienMacro::List&)) );
    connect( interface_, SIGNAL(macroCompleted(AlienMacro)), &commandDialog_->macroModel(), SLOT(remove(AlienMacro)) );

    // command model connections
    connect( interface_, SIGNAL(commandAdded(AlienCommand)), &commandDialog_->commandModel(), SLOT(add(AlienCommand)) );
    connect( interface_, SIGNAL(commandsAdded(const AlienCommand::List&)), &commandDialog_->commandModel(), SLOT(add(const AlienCommand::List&)) );
    connect( interface_, SIGNAL(commandCompleted(AlienCommand)), &commandDialog_->commandModel(), SLOT(remove(AlienCommand)) );

    // fileSystem watcher
    connect( &fileSystemWatcher_, SIGNAL(directoryChangedDelayed(QString)), &localSelectionFrame_->reloadAction(), SLOT(trigger()) );

    // active frame change
    connect( qApp, SIGNAL(focusChanged(QWidget*,QWidget*)), SLOT(_updateActiveFrame(QWidget*,QWidget*)) );

    // clipboard
    _updateClipboard();

    // configuration
    connect( &application, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

    // disable remote items in places widget, untill connected
    _disableRemotePlaces();

    // busy widget
    busyWidget_ = new BusyWidget( main, BusyWidget::BottomRight );
    busyWidget_->hide();

}

//____________________________________________________________________________________________
void MainWindow::initialize( const File& constPath )
{

    File path( Alien::baseName( constPath ) );
    const bool isRemote( Alien::isRemote( constPath ) );
    const bool restoreSession( XmlOptions::get().get<bool>( "RESTORE_SESSION" ) );

    Debug::Throw() << "MainWindow::initialize - path: " << path << " (" << (isRemote ? "remote":"local" ) << ")" << endl;

    // initialize local root path list
    File::List rootPathList;
    foreach( const QFileInfo& fileInfo, QDir::drives() )
    { rootPathList << fileInfo.path(); }

    localSelectionFrame_->setRootPathList( rootPathList );

    // request local home directory
    emit macroRequested( AlienMacro( AlienMacro::HomeDirectory ) << FileInfo( "", FileInfo::Folder ) << FileInfo( "no update" ) );

    // set local path
    if( !isRemote && !path.isEmpty() && ( path = path.expand() ).exists() ) _updatePath( FileInfo( path, FileInfo::Folder ) );
    else if( restoreSession && XmlOptions::get().contains( "LOCAL_PATH" ) ) _updatePath( FileInfo( XmlOptions::get().raw( "LOCAL_PATH" ), FileInfo::Folder ) );
    else _updatePath( FileInfo( Util::workingDirectory(), FileInfo::Folder ) );

    // check proxy and request remote home directory
    AlienMacro::List macros;
    macros
        << AlienMacro( AlienMacro::CheckProxy )
        << ( AlienMacro( AlienMacro::HomeDirectory ) << FileInfo( "", FileInfo::Remote|FileInfo::Folder ) << FileInfo( "no update" ) );
    emit macrosRequested( macros );

    // set remote path
    if( isRemote ) _updatePath( FileInfo( path, FileInfo::Remote|FileInfo::Folder ) );
    else if( restoreSession && XmlOptions::get().contains( "REMOTE_PATH" ) )
    { _updatePath( FileInfo( XmlOptions::get().raw( "REMOTE_PATH" ), FileInfo::Remote|FileInfo::Folder ) ); }

}

//____________________________________________________________________________________________
void MainWindow::updatePath( const File& constPath )
{

    // check path
    if( constPath.isEmpty() ) return;

    // check whether local or remote path is set
    File path( Alien::baseName( constPath ) );
    const bool isRemote( Alien::isRemote( constPath ) );
    Debug::Throw() << "MainWindow::updatePath - path: " << path << " (" << (isRemote ? "remote":"local" ) << ")" << endl;

    // set path
    if( isRemote ) _updatePath( FileInfo( path, FileInfo::Remote|FileInfo::Folder ) );
    else if( (path = path.expand() ).exists() ) _updatePath( FileInfo( path, FileInfo::Folder ) );

    return;
}

//____________________________________________________________________________________________
FileSelectionFrame& MainWindow::activeFrame( void ) const
{ return *(localSelectionFrame_->isActive() ? localSelectionFrame_:remoteSelectionFrame_); }

//____________________________________
void MainWindow::dragEnterEvent( QDragEnterEvent* event )
{
    Debug::Throw( "MainWindow::dragEnterEvent.\n" );
    BaseMainWindow::dragEnterEvent( event );
}

//____________________________________
void MainWindow::dragLeaveEvent( QDragLeaveEvent* event )
{
    Debug::Throw( "MainWindow::dragLeaveEvent.\n" );
    BaseMainWindow::dragLeaveEvent( event );
}


//____________________________________
void MainWindow::dragMoveEvent( QDragMoveEvent* event )
{
    Debug::Throw( "MainWindow::dragMoveEvent.\n" );
    BaseMainWindow::dragMoveEvent( event );
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    event->accept();
    qApp->quit();
}

//_______________________________________________________________________
bool MainWindow::_checkTransfer( const FileInfo& destination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_checkTransfer.\n" );

    // make sure we are not copying file to same location
    if( destination.file() == selectionFrame( destination.location() ).path().file() )
    {

        foreach( const FileInfo& fileInfo, selection )
        {
            if( fileInfo.location() == destination.location() && fileInfo.path() == destination.file() )
            {

                InformationDialog dialog( this, tr( "Items cannot be copied onto themselves. <copy> aborted." ) );
                dialog.setWindowTitle( tr( "Warning - Gridftp" ) );
                dialog.exec();
                return false;
            }
        }

    }

    // make sure one does not copy one file into itself
    foreach( const FileInfo& fileInfo, selection )
    {
        if(
            destination.isFolder() &&
            fileInfo.location() == destination.location() &&
            fileInfo.file() == destination.file() )
        {

            InformationDialog dialog( this, tr( "A folder cannot be copied into itself. <Copy> aborted" ) );
            dialog.setWindowTitle( tr( "Warning - Gridftp" ) );
            dialog.exec();
            return false;

        }

    }

    return true;

}

//_______________________________________________________________________
void MainWindow::_paste( const FileInfo& destination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_paste.\n" );

    // retrieve selected files
    if( selection.empty() ) return;

    // add list recursive commands
    AlienMacro::List macros;
    macros << AlienMacro::Clear;
    foreach( const FileInfo& fileInfo, selection )
    { macros << ( AlienMacro( AlienMacro::ListRecursive ) << fileInfo << destination ); }

    // emit copy command
    const FileInfo source( selection.front().path(), FileInfo::Folder|selection.front().location() );
    macros << ( AlienMacro( AlienMacro::CopyRecursive, AlienMacro::IgnoreDisconnect ) << source << destination );

    // trigger update (not needed if destination is not empty
    if( destination.file() == selectionFrame( destination.location() ).path().file() )
    {
        macros << AlienMacro::Clear;
        macros << ( AlienMacro( AlienMacro::ClearSelection ) << destination );
        macros << ( AlienMacro( AlienMacro::ShowDirectory ) << destination );

        foreach( const FileInfo& fileInfo, _destinationList( destination, selection ) )
        {  macros << ( AlienMacro( AlienMacro::Select ) << fileInfo ); }

    }

    // emit macros
    if( !macros.empty() )
    { emit macrosRequested( macros ); }

}

//_______________________________________________________________________
FileSelectionClipboard MainWindow::_move( const FileInfo& destination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_move.\n" );

    // list of macros
    AlienMacro::List macros;

    // for local-to-local or remote-to-remote move, use Rename command
    if( selection.front().location() == destination.location() )
    {

        // loop over files in clipboard
        foreach( const FileInfo& fileInfo, selection )
        {

            // generate destination file
            FileInfo destinationFile( fileInfo );
            destinationFile.setFile( fileInfo.file().localName().addPath( destination.file() ) );
            if( destination.isLocal() ) destinationFile.setLocal();
            else destinationFile.setRemote();

            // add macro
            macros << ( AlienMacro( AlienMacro::Rename ) << fileInfo << destinationFile );

        }

    } else {

        macros << AlienMacro::Clear;
        foreach( const FileInfo& fileInfo, selection )
        { macros << ( AlienMacro( AlienMacro::ListRecursive ) << fileInfo << destination ); }

        // emit copy command
        const FileInfo source( selection.front().path(), FileInfo::Folder|selection.front().location() );
        macros << ( AlienMacro( AlienMacro::CopyRecursive, AlienMacro::IgnoreDisconnect ) << source << destination );

        // emit remove command
        macros << AlienMacro::RemoveRecursive;

    }

    // generate updated clipboard
    FileSelectionClipboard localClipboard( _destinationList( destination, selection ) );
    localClipboard.setMode( FileSelectionClipboard::Copy );

    // trigger destination update
    if( destination == selectionFrame( destination.type() ).path() )
    {
        macros << AlienMacro::Clear;
        macros << ( AlienMacro( AlienMacro::ShowDirectory ) << destination );

        foreach( const FileInfo& fileInfo, localClipboard )
        { macros << ( AlienMacro( AlienMacro::Select ) << fileInfo ); }

    }

    // decide if source has to be updated at the end of the macro
    const FileInfo sourcePath( selectionFrame( selection.front().type() ).path() );
    foreach( const FileInfo& fileInfo, selection )
    {
        if( fileInfo.path() == sourcePath.file() )
        {
            macros << AlienMacro::Clear;
            macros << ( AlienMacro( AlienMacro::ShowDirectory ) << sourcePath );
            break;
        }
    }

    // emit macros
    if( !macros.empty() ) emit macrosRequested( macros );

    // return local clipboard, for replacement
    return localClipboard;
}

//_______________________________________________________________________
FileInfo::List MainWindow::_destinationList( const FileInfo& destination, const FileInfo::List& selection ) const
{

    FileInfo::List destinationList;
    foreach( const FileInfo& fileInfo, selection )
    {

        // generate destination file
        FileInfo destinationFile( fileInfo );
        destinationFile.setFile( fileInfo.file().localName().addPath( destination.file() ) );
        if( destination.isLocal() ) destinationFile.setLocal();
        else destinationFile.setRemote();
        destinationList << destinationFile;

    }

    return destinationList;

}

//____________________________________________________________________________________________
void MainWindow::_updatePath( const FileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_updatePath - file: " << fileInfo << endl;
    Q_ASSERT( fileInfo.isFolder() );

    // update selection frame
    selectionFrame( fileInfo.type() ).setPath( fileInfo.file() );
    _updateWindowTitle();

    // request directory content
    emit macrosRequested( AlienMacro::List()
        << AlienMacro::Clear
        << (AlienMacro( AlienMacro::ShowDirectory ) << fileInfo ) );

    // also update fileSystemWatcher
    if( fileInfo.isLocal() )
    {
        const QStringList directories( fileSystemWatcher_.directories() );
        if( !directories.isEmpty() ) fileSystemWatcher_.removePaths( directories );
        if( !fileInfo.file().isEmpty() ) fileSystemWatcher_.addPath( fileInfo.file() );
    }

    return;
}

//____________________________________________________________________________________________
void MainWindow::_updateViewMode( FileSelectionFrame::ViewMode mode )
{
    FileSelectionFrame& selectionFrame( activeFrame() );
    selectionFrame.setViewMode( mode );
    selectionFrame.setShowNavigator( showNavigator_ && mode == FileSelectionFrame::List );
}

//____________________________________________________________________________________________
void MainWindow::_updateActiveFrame( QWidget*, QWidget* widget )
{
    if( localSelectionFrame_->owns( widget ) )
    {

        localSelectionFrame_->setActive( true );
        remoteSelectionFrame_->setActive( false );
        _updateNavigationActions();
        _updateWindowTitle();

    } else if(  remoteSelectionFrame_->owns( widget ) ) {

        localSelectionFrame_->setActive( false );
        remoteSelectionFrame_->setActive( true );
        _updateNavigationActions();
        _updateWindowTitle();

    }

}

//______________________________________________________
void MainWindow::_updateWindowTitle( void )
{

    File path( activeFrame().path().file() );
    if( path.endsWith( '/' ) ) path = path.left( path.size()-1 );

    QString buffer;
    QTextStream what( &buffer );
    if( !path.isEmpty() ) what << path.localName() << " - ";
    what << "Gridftp";
    setWindowTitle( buffer );

}

//___________________________________________________________________________
void MainWindow::_disableRemotePlaces( void )
{
    Debug::Throw( "MainWindow::_disableRemotePlaces.\n" );
    bool changed( false );
    foreach( const BaseFileInfo& fileInfo, placesWidget_->items() )
    { if( fileInfo.isRemote() ) changed |= placesWidget_->setItemIsValid( fileInfo, false ); }

    if( changed ) placesWidget_->update();

}

//___________________________________________________________________________
void MainWindow::_requestPlacesUpdate( int type )
{

    Debug::Throw( "MainWindow::_requestPlacesUpdate.\n" );

    // loop over places widget local files and check existence
    AlienMacro::List macros;
    macros << AlienMacro::Clear;

    foreach( const BaseFileInfo& fileInfo, placesWidget_->items() )
    {

        // check file location against type
        if( type&FileInfo::Remote ) { if( fileInfo.isLocal() ) continue; }
        else if( fileInfo.isRemote() ) continue;

        macros << ( AlienMacro( AlienMacro::Add ) << FileInfo( fileInfo ) );
    }

    macros << AlienMacro::CheckFilesExist;
    emit macrosRequested( macros );

}


//____________________________________________________________________________________________
void MainWindow::_connectionStateChanged( bool connected )
{
    if( connected )
    {

        wasConnected_ = true;
        _requestRemotePlacesUpdate();

    } else _disableRemotePlaces();
}

//____________________________________________________________________________________________
void MainWindow::_updateHome( const FileInfo& fileInfo, bool alsoUpdate )
{
    Debug::Throw() << "MainWindow::_updateHome - file: " << fileInfo << endl;
    Q_ASSERT( fileInfo.isFolder() );

    // update selection frame
    selectionFrame( fileInfo.type() ).setHomePath( fileInfo.file() );
    if( alsoUpdate ) _updatePath( fileInfo );

}

//____________________________________________________________________________________________
void MainWindow::_updateContent( const FileInfo& path, const FileInfo::List& files )
{

    FileSelectionFrame& selectionFrame( this->selectionFrame( path.type() ) );

    // check path and do nothing if does not match
    if( selectionFrame.path().file() != path.file() ) return;

    // store
    XmlOptions::get().set( path.isLocal() ? "LOCAL_PATH":"REMOTE_PATH", path.file() );

    // build file info list with local filenames
    FileInfo::List local;
    foreach( const FileInfo& fileInfo, files )
    { if( fileInfo.isLocal() == path.isLocal() ) local << fileInfo; }

    // update selection frame
    if( selectionFrame.isDirty() )
    {

        selectionFrame.model().clear();
        selectionFrame.model().set( local );
        selectionFrame.setDirty( false );

    } else {

        selectionFrame.model().update( local );

    }

    // navigator
    selectionFrame.setShowNavigator( showNavigator_ && selectionFrame.viewMode() == FileSelectionFrame::List );

}

//____________________________________________________________________________________________
void MainWindow::_addFileInfo( const FileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_addFileInfo - " << fileInfo << endl;
    FileSelectionFrame& selectionFrame( this->selectionFrame( fileInfo.type() ) );
    if( selectionFrame.path().file() != fileInfo.path() ) return;
    selectionFrame.model().add( fileInfo );

    if( fileInfo.isRemote() && fileInfo.isFolder() && placesWidget_->setItemIsValid( fileInfo, true ) )
    { placesWidget_->update(); }

}


//____________________________________________________________________________________________
void MainWindow::_removeFileInfo( const FileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_removeFileInfo - " << fileInfo << endl;
    FileSelectionFrame& selectionFrame( this->selectionFrame( fileInfo.type() ) );
    if( selectionFrame.path().file() != fileInfo.path() ) return;
    selectionFrame.model().remove( fileInfo );

    if( fileInfo.isRemote() && fileInfo.isFolder() && placesWidget_->setItemIsValid( fileInfo, false ) )
    { placesWidget_->update(); }

}

//____________________________________________________________________________________________
void MainWindow::_macroStarted( const AlienMacro& macro )
{

    Debug::Throw( "MainWindow::_macroStarted.\n" );
    switch( macro.type() )
    {
        case AlienMacro::CopyRecursive:
        case AlienMacro::RemoveRecursive:
        statusbar_->showProgressBar();
        break;

        default: break;
    }

}

//____________________________________________________________________________________________
void MainWindow::_macroCompleted( const AlienMacro& macro )
{

    Debug::Throw( "MainWindow::_macroCompleted.\n" );
    switch( macro.type() )
    {

        case AlienMacro::CopyRecursive:
        case AlienMacro::RemoveRecursive:
        statusbar_->hideProgressBar();
        break;

        case AlienMacro::ClearSelection:
        selectionFrame( macro.argument(0).type() ).clearSelection();
        break;

        case AlienMacro::Select:
        {
            const FileInfo fileInfo( macro.argument(0) );
            selectionFrame( fileInfo.type() ).select( fileInfo );
            break;
        }

        case AlienMacro::CheckFilesExist:
        {

            // retrieve files and check if copy has succedded
            bool changed( false );
            foreach( const FileInfo& fileInfo, interface_->filesRecursive() )
            { changed |= placesWidget_->setItemIsValid( fileInfo, fileInfo.isSuccess() ); }

            if( changed ) placesWidget_->update();

            break;
        }

        default: break;
    }

}

//____________________________________________________________________________________________
void MainWindow::_lostConnection( const QString& details )
{
    Debug::Throw( "MainWindow::_lostConnection.\n" );
    QString message( wasConnected_ ?
        tr( "Lost connection to remote file system.\nTry to reconnect ?" ):
        tr( "No connection to remote file system. Try connect ?" ) );
    QuestionDialog dialog( this, message );
    dialog.setOptionName( "CONNECTION_LOST_DIALOG" );
    dialog.setDetails( details );
    dialog.okButton().setText( tr( "Connect" ) );
    dialog.okButton().setIcon( IconEngine::get( IconNames::Connect ) );

    if( dialog.exec() ) _connect();

    return;
}

//____________________________________________________________________________________________
void MainWindow::_commandsFailed( const AlienCommand::List& commands )
{

    if( commands.isEmpty() ) return;
    Debug::Throw() << "MainWindow::_commandsFailed - " << commands.front().name() << endl;

    if( commands.size() == 1 && commands.front().type() == AlienCommand::CheckProxy )
    {

        WarningDialog dialog( this, commands.front().errorString() );
        dialog.setDetails( commands.front().errorDetails() );
        dialog.setWindowTitle( tr( "Error - Gridftp" ) );
        dialog.exec();
        interface_->clearFailedCommands();

    } else {

        CommandsFailedDialog dialog( this, commands );
        dialog.setWindowTitle( tr( "Error - Gridftp" ) );
        if( !dialog.exec() ) { interface_->clearFailedCommands(); }


    }

    return;

}

//______________________________________________________
void MainWindow::_updateVisibilityActions( void )
{

    Debug::Throw( "MainWindow::_updateVisibilityActions.\n" );
    localSelectionFrame_->visibilityAction().setEnabled( remoteSelectionFrame_->visibilityAction().isChecked() );
    remoteSelectionFrame_->visibilityAction().setEnabled( localSelectionFrame_->visibilityAction().isChecked() );

    // handle focus
    if( !localSelectionFrame_->visibilityAction().isChecked() ) remoteSelectionFrame_->setFocus();
    if( !remoteSelectionFrame_->visibilityAction().isChecked() ) localSelectionFrame_->setFocus();
}

//______________________________________________________
void MainWindow::_updateNavigationActions( void )
{
    Debug::Throw( "MainWindow::_updateNavigationActions.\n" );
    reloadAction_->setEnabled( activeFrame().reloadAction().isEnabled() );

    previousDirectoryButton_->setEnabled( activeFrame().previousDirectoryAction().isEnabled() );
    previousDirectoryButton_->setMenu( &activeFrame().pathEditor().previousPathMenu() );

    nextDirectoryButton_->setEnabled( activeFrame().nextDirectoryAction().isEnabled() );
    nextDirectoryButton_->setMenu( &activeFrame().pathEditor().nextPathMenu() );

    parentDirectoryAction_->setEnabled( activeFrame().parentDirectoryAction().isEnabled() );

    return;
}

//____________________________________________________________________________________________
void MainWindow::_fileSelected( const FileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_fileSelected - file: " << fileInfo << endl;
    if( fileInfo.isFolder() ) _updatePath( fileInfo );
    else _openFiles( FileInfo::List() << fileInfo );
    return;
}

//____________________________________________________________________________________________
void MainWindow::_placeSelected( const BaseFileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_placeSelected - file: " << fileInfo << endl;
    if( !fileInfo.isFolder() ) return;

    // update path
    _updatePath( fileInfo );

    // set focus to correct selection frame
    selectionFrame( fileInfo.type() ).setFocus();

    return;
}

//_______________________________________________________________________
void MainWindow::_connect( void )
{
    Debug::Throw( "MainWindow::_connect.\n" );
    AlienMacro::List macros;
    macros
        << AlienMacro::Connect
        << ( AlienMacro( AlienMacro::HomeDirectory ) << FileInfo( "", FileInfo::Remote|FileInfo::Folder ) << FileInfo( "no update" ) )
        << ( AlienMacro( AlienMacro::ShowDirectory ) << remoteSelectionFrame_->path() );

    emit macrosRequested( macros );
}

//_______________________________________________________________________
void MainWindow::_reload( void )
{ if( activeFrame().reloadAction().isEnabled() ) activeFrame().reloadAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_previousDirectory( void )
{ if( activeFrame().previousDirectoryAction().isEnabled() ) activeFrame().previousDirectoryAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_nextDirectory( void )
{ if( activeFrame().nextDirectoryAction().isEnabled() ) activeFrame().nextDirectoryAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_parentDirectory( void )
{ if( activeFrame().parentDirectoryAction().isEnabled() ) activeFrame().parentDirectoryAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_homeDirectory( void )
{ emit macroRequested( AlienMacro( AlienMacro::HomeDirectory ) << FileInfo( "", activeFrame().type()|FileInfo::Folder ) ); }

//_______________________________________________________________________
void MainWindow::_updateClipboard( const FileSelectionClipboard& clipboard )
{
    Debug::Throw( "MainWindow::_updateClipboard.\n" );
    clipboard_ = clipboard;

    foreach( FileSelectionFrame* frame, selectionFrames() )
    {

        // pass clipboard to models
        frame->model().setClipboard( clipboard );

        if( clipboard_.empty() )
        {

            frame->pasteInFolderAction().setEnabled( false );
            frame->pasteAction().setEnabled( false );
            frame->pasteAction().setText( tr( "Paste" ) );

        } else {

            frame->pasteInFolderAction().setEnabled( true );
            frame->pasteAction().setEnabled( true );
            frame->pasteAction().setText( QString( tr( "Paste %1" ) ).arg( clipboard_.description() ) );

        }
    }

}

//_______________________________________________________________________
void MainWindow::_pasteSelection( const FileInfo& constDestination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_pasteSelection.\n" );
    if( selection.empty() ) return;

    // make sure valid destination is set
    FileInfo destination( constDestination );
    if( destination.file().isEmpty() ) destination = selectionFrame( destination.location() ).path();

    // check transfer
    if( !_checkTransfer( destination, selection ) ) return;

    // open menu
    QMenu menu( this );
    menu.addActions( menuActions_ );
    menu.ensurePolished();
    QAction* action( menu.exec( QCursor::pos() ) );
    if( action == menuActions_[0] ) _paste( destination, selection );
    else if( action == menuActions_[1] ) _move( destination, selection );
    else return;

}

//_______________________________________________________________________
void MainWindow::_pasteClipboard( const FileInfo& destination )
{

    Debug::Throw( "MainWindow::_pasteClipboard.\n" );
    if( clipboard_.empty() ) return;
    if( !_checkTransfer( destination, clipboard_ ) ) return;

    // perform command
    if( clipboard_.mode() == FileSelectionClipboard::Copy ) _paste( destination, clipboard_ );
    else clipboard_  = _move( destination, clipboard_ );

}

//____________________________________________________________________________________________
void MainWindow::_openFiles( const FileInfo::List& files )
{

    Debug::Throw( "MainWindow::_openFiles.\n" );
    if( files.empty() ) return;

    // ask confirmation
    OpenFilesDialog dialog( this, files );
    dialog.setWindowTitle( tr( "Open Files - Gridftp" ) );

    // retrieve applications from options
    foreach( const Option& application, XmlOptions::get().specialOptions( "APPLICATIONS" ) )
    { dialog.actionComboBox().addItem( application.raw() ); }

    if( !dialog.exec() ) return;

    // retrieve applications from combobox and add as options
    QString command( dialog.actionComboBox().currentText() );
    if( command.isEmpty() )
    {
        InformationDialog( this, tr( "No command specified to open the selected files. <Open> canceled." ) ).exec();
        return;
    }

    // update options
    XmlOptions::get().add( "APPLICATIONS", Option( command, Option::Recordable|Option::Current ) );

    AlienMacro::List macros;
    foreach( const FileInfo& fileInfo, files )
    {

        if( fileInfo.isLocal() )
        {

            macros
                << (AlienMacro( AlienMacro::Open )
                << fileInfo
                << command );

        } else {

            // generate scratch file name
            File relativeName = QDir::root().relativeFilePath( fileInfo.file() );

            // temporary path
            emit scratchFileCreated( Util::tmp() + "/gridftp" );
            emit scratchFileCreated( Util::tmp() + "/gridftp/" + Util::user() );

            // generate destination
            FileInfo temporary(  Util::tmp() + "/gridftp/" + Util::user(), FileInfo::Folder );
            FileInfo destination( fileInfo );
            destination.setFile( relativeName.addPath( temporary.file() ) );
            destination.setLocal();

            // make sure local path exists
            if( !destination.path().exists() )
            {
                macros << ( AlienMacro( AlienMacro::CreateDirectory ) << FileInfo( destination.path(), FileInfo::Folder ) );
                emit scratchFileCreated( destination.path() );
            }

            // register local file to scratch manager
            emit scratchFileCreated( destination.file() );

            // generate copy command
            macros
                << ( AlienMacro( AlienMacro::Copy )
                << fileInfo
                << destination );

            macros
                << (AlienMacro( AlienMacro::Open )
                << destination
                << command );
        }

    }

    // emit macros
    if( !macros.empty() ) emit macrosRequested( macros );

}

//_______________________________________________
void MainWindow::_findCollection( const FileInfo& path )
{
    Debug::Throw( "MainWindow::_findCollection.\n" );
    FindCollectionDialog dialog;
    dialog.setPath( path.file() );
    connect( &dialog, SIGNAL(macroRequested(AlienMacro)), interface_, SLOT(addMacro(AlienMacro)) );
    connect( interface_, SIGNAL(collectionAvailable(QString)), &dialog, SLOT(processCollection(QString)) );

    if( localSelectionFrame_->isVisible() )
    { dialog.setDefaultFilename( localSelectionFrame_->path().file() ); }

    dialog.setWindowTitle( tr( "Find Collection - Gridftp" ) );
    dialog.exec();
}

//_______________________________________________
void MainWindow::_fileInformation( const FileInfo& fileInfo, const QIcon& icon )
{

    FileInformationDialog dialog( this, fileInfo );
    dialog.setIcon( icon );
    connect( &dialog, SIGNAL(macroRequested(AlienMacro)), interface_, SLOT(addMacro(AlienMacro)) );
    connect( &dialog, SIGNAL(macrosRequested(const AlienMacro::List&)), interface_, SLOT(addMacros(const AlienMacro::List&)) );
    connect( &dialog, SIGNAL(requestAbort()), interface_, SLOT(abortCurrent()) );

    connect( interface_, SIGNAL(locationAvailable(FileInfo,QStringList)), &dialog, SLOT(updateLocation(FileInfo,QStringList)) );
    connect( interface_, SIGNAL(sizeAvailable(FileInfo,qint64)), &dialog, SLOT(updateSize(FileInfo,qint64)) );
    connect( interface_, SIGNAL(macroStarted(AlienMacro)), &dialog, SLOT(macroStarted(AlienMacro)) );
    connect( interface_, SIGNAL(macroCompleted(AlienMacro)), &dialog, SLOT(macroCompleted(AlienMacro)) );

    if( fileInfo.isRemote() && fileInfo.isDocument() ) dialog.requestLocation();

    dialog.exec();

}

//____________________________________________________________________________________________
void MainWindow::_toggleShowNavigator( bool value )
{

    Debug::Throw( "MainWindow::_toggleShowNavigator.\n" );
    if( showNavigator_ == value ) return;
    showNavigator_ = value;
    foreach( FileSelectionFrame* selectionFrame, selectionFrames() )
    { selectionFrame->setShowNavigator( showNavigator_ && selectionFrame->viewMode() == FileSelectionFrame::List ); }

}

//_______________________________________________
void MainWindow::_toggleShowHiddenFiles( bool value )
{
    Debug::Throw( "MainWindow::_toggleShowHiddenFiles.\n" );

    interface_->setShowHiddenFiles( value );
    foreach( FileSelectionFrame* frame, selectionFrames() )
    { frame->setShowHiddenFiles( value ); }

    XmlOptions::get().set<bool>("SHOW_HIDDEN_FILES", value );

}

//_______________________________________________
void MainWindow::_showCommandDialog( void )
{

    Debug::Throw( "MainWindow::_showCommandDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );
    commandDialog_->show();
    commandDialog_->raise();

}

//_______________________________________________
void MainWindow::_showLog( void )
{
    Debug::Throw( "MainWindow::_showLog.\n" );
    logDialog_->show();
    logDialog_->toggleSticky( false );
    logDialog_->raise();
    XmlOptions::get().set<bool>( "SHOW_LOG", true );
}

//_______________________________________________
void MainWindow::_toggleLog( bool value )
{
    Debug::Throw( "MainWindow::_toggleLog.\n" );
    if( value )
    {

        connect( interface_, SIGNAL(logAvailable(QString,Format::TextFormatFlags)), &logDialog_->editor(), SLOT(append(QString,Format::TextFormatFlags)), Qt::UniqueConnection );

    } else {

        disconnect( interface_, SIGNAL(logAvailable(QString,Format::TextFormatFlags)), &logDialog_->editor(), SLOT(append(QString,Format::TextFormatFlags)) );
        logDialog_->hide();
        logDialog_->editor().clear();

    }

    logAction_->setEnabled( value );
    XmlOptions::get().set<bool>( "LOG_ENABLED", value );

}

//_______________________________________________
void MainWindow::_logDialogHidden( void )
{
    Debug::Throw( "MainWindow::_logDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
}

//_______________________________________________
void MainWindow::_commandDialogHidden( void )
{
    Debug::Throw( "MainWindow::_commandDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//___________________________________________________________________________
void MainWindow::_requestPlacesUpdate( const BaseFileInfo& fileInfo )
{

    Debug::Throw( "MainWindow::_requestPlacesUpdate.\n" );

    // loop over places widget local files and check existence
    AlienMacro::List macros;
    macros << AlienMacro::Clear;
    macros << ( AlienMacro( AlienMacro::Add ) << FileInfo( fileInfo ) );
    macros << AlienMacro::CheckFilesExist;
    emit macrosRequested( macros );

}

//____________________________________________________________________________________________
void MainWindow::_updateConfiguration( void )
{
    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // navigator
    _toggleShowNavigator( XmlOptions::get().get<bool>( "SHOW_NAVIGATOR" ) );

    // hidden files
    showHiddenFilesAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_HIDDEN_FILES" ) );

    // clear cached icons
    iconProvider_->clear();

    // log and command list
    toggleLogAction().setChecked( XmlOptions::get().get<bool>( "LOG_ENABLED" ) );
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandDialog();
    if( XmlOptions::get().get<bool>( "SHOW_LOG" ) ) _showLog();
    else logDialog_->hide();

}

//_____________________________________________
void MainWindow::_installActions( void )
{

    Debug::Throw( "FileSelectionFrame::_installActions.\n" );

    // connect
    addAction( connectAction_ = new QAction( IconEngine::get( IconNames::Connect ), tr( "Connect..." ), this ) );
    connect( connectAction_, SIGNAL(triggered()), SLOT(_connect()) );

    // home directory
    addAction( homeDirectoryAction_ = new QAction( IconEngine::get( IconNames::Home ), tr( "Home" ), this ) );
    connect( homeDirectoryAction_, SIGNAL(triggered()), SLOT(_homeDirectory()) );

    // previous directory
    previousDirectoryButton_ = new CustomToolButton( this );
    previousDirectoryButton_->setIcon( IconEngine::get( IconNames::PreviousDirectory ) );
    previousDirectoryButton_->setText( tr( "Back" ) );
    previousDirectoryButton_->setToolTip( tr( "Change path to previous directory (from history)" ) );
    connect( previousDirectoryButton_, SIGNAL(clicked()), SLOT(_previousDirectory()) );

    // next directory
    nextDirectoryButton_ = new CustomToolButton( this );
    nextDirectoryButton_->setIcon( IconEngine::get( IconNames::NextDirectory ) );
    nextDirectoryButton_->setText( tr( "Forward" ) );
    nextDirectoryButton_->setToolTip( tr( "Change path to next directory (from history)" ) );
    connect( nextDirectoryButton_, SIGNAL(clicked()), SLOT(_nextDirectory()) );

    // parent directory in tree
    addAction( parentDirectoryAction_ = new QAction( IconEngine::get( IconNames::Parent ), tr( "Parent Directory" ), this ) );
    parentDirectoryAction_->setIconText( tr( "Up" ) );
    connect( parentDirectoryAction_, SIGNAL(triggered()), SLOT(_parentDirectory()) );

    // reload
    addAction( reloadAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Reload" ), this ) );
    connect( reloadAction_, SIGNAL(triggered()), SLOT(_reload()) );
    reloadAction_->setShortcut( QKeySequence::Refresh );

    // abort
    addAction( abortAction_ = new QAction( IconEngine::get( IconNames::Abort ), tr( "Abort" ), this ) );
    connect( abortAction_, SIGNAL(triggered()), interface_, SLOT(abortAll()) );

    // show hidden files
    showHiddenFilesAction_ = new QAction( tr( "Show Hidden Files" ), this );
    showHiddenFilesAction_->setCheckable( true );
    showHiddenFilesAction_->setChecked( false );
    connect( showHiddenFilesAction_, SIGNAL(toggled(bool)), SLOT(_toggleShowHiddenFiles(bool)) );

    // command list
    commandDialogAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Show Command List" ), this );
    connect( commandDialogAction_, SIGNAL(triggered()), SLOT(_showCommandDialog()) );

    // log
    logAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Show Log" ), this );
    logAction_->setEnabled( false );
    connect( logAction_, SIGNAL(triggered()), SLOT(_showLog()) );

    toggleLogAction_ = new QAction( tr( "Enable Log" ), this );
    toggleLogAction_->setCheckable( true );
    toggleLogAction_->setChecked( false );
    connect( toggleLogAction_, SIGNAL(toggled(bool)), SLOT(_toggleLog(bool)) );

    // menu actions
    QAction* action;
    menuActions_ << ( action = new QAction( IconEngine::get( IconNames::Copy ), tr( "Copy Here" ), this ) );
    menuActions_ << ( action = new QAction( IconEngine::get( IconNames::Move ), tr( "Move Here" ), this ) );
    menuActions_ << ( action = new QAction( this ) );
    action->setSeparator( true );
    menuActions_ << ( action = new QAction( IconEngine::get( IconNames::DialogCancel ), tr( "Cancel" ), this ) );
    action->setShortcut( Qt::Key_Escape );

}
